@@ -104,21 +104,25 @@
 #include "gdata-types.h"
 #include "gdata-private.h"
 #include "gdata-access-handler.h"
+#include "gdata-documents-access-rule.h"
 #include "gdata-documents-service.h"
 
 #include "gdata-documents-spreadsheet.h"
 #include "gdata-documents-presentation.h"
 #include "gdata-documents-text.h"
 #include "gdata-documents-folder.h"
+#include "gdata-documents-utils.h"
 
 static void gdata_documents_entry_access_handler_init (GDataAccessHandlerIface *iface);
 static void gdata_documents_entry_finalize (GObject *object);
 static void gdata_entry_dispose (GObject *object);
+static const gchar *get_content_type (void);
 static void get_namespaces (GDataParsable *parsable, GHashTable *namespaces);
 static void gdata_documents_entry_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_documents_entry_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
 static gboolean parse_json (GDataParsable *parsable, JsonReader *reader, gpointer user_data, GError **error);
 static gboolean post_parse_json (GDataParsable *parsable, gpointer user_data, GError **error);
+static void get_json (GDataParsable *parsable, JsonBuilder *builder);
 static gchar *get_entry_uri (const gchar *id);
 
 struct _GDataDocumentsEntryPrivate {
@@ -161,6 +165,8 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 
 	parsable_class->parse_json = parse_json;
 	parsable_class->post_parse_json = post_parse_json;
+	parsable_class->get_content_type = get_content_type;
+	parsable_class->get_json = get_json;
 	parsable_class->get_namespaces = get_namespaces;
 
 	entry_class->get_entry_uri = get_entry_uri;
@@ -300,11 +306,49 @@ get_authorization_domain (GDataAccessHandler *self)
 	return gdata_documents_service_get_primary_authorization_domain ();
 }
 
+static GDataFeed *
+get_rules (GDataAccessHandler *self,
+	   GDataService *service,
+	   GCancellable *cancellable,
+	   GDataQueryProgressCallback progress_callback,
+	   gpointer progress_user_data,
+	   GError **error)
+{
+	GDataAccessHandlerIface *iface;
+	GDataAuthorizationDomain *domain = NULL;
+	GDataFeed *feed;
+	GDataLink *_link;
+	SoupMessage *message;
+
+	_link = gdata_entry_look_up_link (GDATA_ENTRY (self), GDATA_LINK_ACCESS_CONTROL_LIST);
+	g_assert (_link != NULL);
+
+	iface = GDATA_ACCESS_HANDLER_GET_IFACE (self);
+	if (iface->get_authorization_domain != NULL) {
+		domain = iface->get_authorization_domain (self);
+	}
+
+	message = _gdata_service_query (service, domain, gdata_link_get_uri (_link), NULL, cancellable, error);
+	if (message == NULL) {
+		return NULL;
+	}
+
+	g_assert (message->response_body->data != NULL);
+
+	feed = _gdata_feed_new_from_json (GDATA_TYPE_FEED, message->response_body->data, message->response_body->length, GDATA_TYPE_DOCUMENTS_ACCESS_RULE,
+					  progress_callback, progress_user_data, error);
+
+	g_object_unref (message);
+
+	return feed;
+}
+
 static void
 gdata_documents_entry_access_handler_init (GDataAccessHandlerIface *iface)
 {
 	iface->is_owner_rule = is_owner_rule;
 	iface->get_authorization_domain = get_authorization_domain;
+	iface->get_rules = get_rules;
 }
 
 static void
@@ -750,6 +794,74 @@ post_parse_json (GDataParsable *parsable, gpointer user_data, GError **error)
 	return TRUE;
 }
 
+static const gchar *
+get_content_type (void)
+{
+	return "application/json";
+}
+
+static void
+get_json (GDataParsable *parsable, JsonBuilder *builder)
+{
+	GList *categories;
+	GList *i;
+	GList *parent_folders_list;
+
+	GDATA_PARSABLE_CLASS (gdata_documents_entry_parent_class)->get_json (parsable, builder);
+
+	/* Inserting files: https://developers.google.com/drive/v2/reference/files/insert */
+
+	categories = gdata_entry_get_categories (GDATA_ENTRY (parsable));
+	for (i = categories; i != NULL; i = i->next) {
+		GDataCategory *category = GDATA_CATEGORY (i->data);
+		const gchar *label;
+		const gchar *scheme;
+
+		label = gdata_category_get_label (category);
+		scheme = gdata_category_get_scheme (category);
+		if (label != NULL && label[0] != '\0' && g_strcmp0 (scheme, "http://schemas.google.com/g/2005#kind") == 0) {
+			json_builder_set_member_name (builder, "mimeType");
+			json_builder_add_string_value (builder, label);
+			break;
+		}
+	}
+
+	/* Upload to a folder: https://developers.google.com/drive/web/folder */
+
+	json_builder_set_member_name (builder, "parents");
+	json_builder_begin_array (builder);
+
+	parent_folders_list = gdata_entry_look_up_links (GDATA_ENTRY (parsable), GDATA_LINK_PARENT);
+	for (i = parent_folders_list; i != NULL; i = i->next) {
+		GDataLink *_link = GDATA_LINK (i->data);
+		const gchar *uri;
+		gsize uri_prefix_len;
+
+		/* HACK: Extract the ID from the GDataLink:uri by removing the prefix. Ignore links which
+		 * don't have the prefix. */
+		uri = gdata_link_get_uri (_link);
+		uri_prefix_len = strlen (GDATA_DOCUMENTS_URI_PREFIX);
+		if (g_str_has_prefix (uri, GDATA_DOCUMENTS_URI_PREFIX)) {
+			const gchar *id;
+
+			id = uri + uri_prefix_len;
+			if (id[0] != '\0') {
+				json_builder_begin_object (builder);
+				json_builder_set_member_name (builder, "kind");
+				json_builder_add_string_value (builder, "drive#fileLink");
+				json_builder_set_member_name (builder, "id");
+				json_builder_add_string_value (builder, id);
+				json_builder_end_object (builder);
+			}
+		}
+	}
+
+	json_builder_end_array (builder);
+
+	g_list_free (parent_folders_list);
+
+}
+
 static void
 get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 {
