@@ -1354,13 +1354,16 @@ main (int argc, char *argv[])
 	GDataService *service = NULL;  /* owned */
 	GDataService *unauthorised_service = NULL;  /* owned */
 	GFile *trace_directory = NULL;  /* owned */
+	gchar *path = NULL;
 
 	gdata_test_init (argc, argv);
 
 	mock_server = gdata_test_get_mock_server ();
 	g_signal_connect (G_OBJECT (mock_server), "notify::resolver",
 	                  (GCallback) mock_server_notify_resolver_cb, NULL);
-	trace_directory = g_file_new_for_path (TEST_FILE_DIR "traces/tasks");
+	path = g_test_build_filename (G_TEST_DIST, "traces/tasks", NULL);
+	trace_directory = g_file_new_for_path (path);
+	g_free (path);
 	uhm_server_set_trace_directory (mock_server, trace_directory);
 	g_object_unref (trace_directory);
 
